@@ -3,6 +3,7 @@ from unittest import TestCase
 
 import mock
 from nose.tools import eq_, raises
+import six
 
 from . import Receiver, Sender
 from .exc import (AlreadyProcessed,
@@ -14,6 +15,7 @@ from .exc import (AlreadyProcessed,
                   TokenExpired)
 from .util import (parse_authorization_header,
                    utc_now,
+                   calculate_ts_mac,
                    validate_credentials)
 
 
@@ -304,6 +306,12 @@ class TestSender(Base):
         parsed = parse_authorization_header(sn.request_header)
         eq_(parsed['ext'], "new line \n in the middle")
 
+    def test_ext_with_equality_sign(self):
+        sn = self.Sender(ext="foo=bar")
+        self.receive(sn.request_header)
+        parsed = parse_authorization_header(sn.request_header)
+        eq_(parsed['ext'], "foo=bar")
+
     @raises(BadHeaderValue)
     def test_ext_with_illegal_chars(self):
         self.Sender(ext="something like \t is illegal")
@@ -461,7 +469,16 @@ class TestReceiver(Base):
 
         with mock.patch('mohawk.base.utc_now') as fn:
             fn.return_value = 0  # force an expiry
-            self.sender.accept_response(hdr, content='', content_type='')
+            try:
+                self.sender.accept_response(hdr, content='', content_type='')
+            except TokenExpired:
+                etype, exc, tb = sys.exc_info()
+                hdr = parse_authorization_header(exc.www_authenticate)
+                calculated = calculate_ts_mac(fn(), self.credentials)
+                if isinstance(calculated, six.binary_type):
+                    calculated = calculated.decode('ascii')
+                eq_(hdr['tsm'], calculated)
+                raise
 
     def test_respond_with_bad_ts_skew_ok(self):
         now = utc_now() - 120
